@@ -2,6 +2,17 @@
 
 """
 Test for cpu.py
+
+results format:
+
+    sched   cpu client MB/s
+
+    cfs 45.00%  13254.669841
+    cfs 40.00%  11822.441399
+    cfs 30.00%  5112.963009
+    cfs 20.00%  3449.090009
+    cfs 10.00%  2271.741564
+
 """
 
 import unittest
@@ -16,18 +27,20 @@ class testCPU( unittest.TestCase ):
     def testCPU( self ):
         "Verify that CPU utilization is monotonically decreasing for each scheduler"
         p = pexpect.spawn( 'python -m mininet.examples.cpu' )
-        opts = [ '([a-z]+)\t([\d\.]+)%\t([\d\.]+)', pexpect.EOF ]
+        # matches each line from results( shown above )
+        opts = [ '([a-z]+)\t([\d\.]+)%\t([\d\.]+)',
+                 pexpect.EOF ]
         scheds = []
         while True:
             index = p.expect( opts, timeout=600 )
             if index == 0:
-                sched = p.match.group( 1 ) 
+                sched = p.match.group( 1 )
                 cpu = float( p.match.group( 2 ) )
                 bw = float( p.match.group( 3 ) )
                 if sched not in scheds:
                     scheds.append( sched )
-                    previous_bw = 10 ** 4 # 10 GB/s
-                self.assertTrue( bw < previous_bw )
+                else:
+                    self.assertTrue( bw < previous_bw )
                 previous_bw = bw
             else:
                 break
